module PlanetExpressExpress
  require "date"
  class ShipmentReader
    def initialize(prices, shipment_pool: ShipmentPool.new(size: 10))
      @shipment_pool = shipment_pool
      @prices = prices
      @id = 0
    end

    # Expects to read a file
    def read_shipments(io)
      shipments = {
        S: [],
        M: [],
        L: [],
        all: []
      }

      current_month = 0
      current_year = 0

      io.each_line do |line|
        shipment = parse(line)

        if shipment.date.year != current_year || shipment.date.month != current_month
          yield shipments

          @shipment_pool.put_back(shipments[:all])
          shipments.each do |_,v|
            v.clear
          end
        end
        shipments[shipment.size].push(shipment)
        shipments[:all].push(shipment)

        current_year = shipment.date.year
        current_month = shipment.date.month
      end

      yield shipments
    end

    private

    attr_reader :prices

    def parse(line)
      parts = line.split(" ")

      shipment = @shipment_pool.get

      shipment.date = Date.strptime(parts[0], "%Y-%m-%d")
      shipment.size = parts[1]&.to_sym
      shipment.provider = parts[2]&.to_sym
      shipment.price = 0
      shipment.discount = 0

      shipment.price = prices[
        [
          shipment.provider, shipment.size
        ]
      ]

      shipment
    end
  end
end
